package com.persagy.dmp.rwd.delivery.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.dmp.codetable.entity.CodeTableData;
import com.persagy.dmp.common.constant.ValidEnum;
import com.persagy.dmp.common.model.entity.BaseEntity;
import com.persagy.dmp.define.entity.ObjectInfoDefine;
import com.persagy.dmp.delivery.dto.ObjectInfoCollectDTO;
import com.persagy.dmp.delivery.entity.ObjectInfoCollect;
import com.persagy.dmp.rwd.define.service.IObjectInfoService;
import com.persagy.dmp.rwd.delivery.dao.ObjectInfoCollectMapper;
import com.persagy.dmp.rwd.delivery.service.IObjectInfoCollectService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 项目，现场交付范围管理-信息点
 * @author:linhuili
 * @date:2021/10/13
 */
@Service
public class ObjectInfoCollectServiceImpl extends ServiceImpl<ObjectInfoCollectMapper, ObjectInfoCollect> implements IObjectInfoCollectService {

    @Autowired
    private ObjectInfoCollectMapper objectInfoCollectMapper;

    @Autowired
    private IObjectInfoService objectInfoService;

    /**
     * 数量统计
     * @param queryWrapper 查询条件
     * @return
     */
    @Override
    public Integer queryCount(Wrapper<ObjectInfoCollect> queryWrapper) {
        return objectInfoCollectMapper.selectCount(queryWrapper);
    }

    /**
     * 查询采集的信息点
     * @param queryWrapper 查询条件
     * @return
     */
    @Override
    public List<ObjectInfoCollect> queryByCondition(Wrapper<ObjectInfoCollect> queryWrapper) {
        return objectInfoCollectMapper.selectList(queryWrapper);
    }

    /**
     * 分页询采集的信息点
     * @param page 分页条件
     * @param queryWrapper 查询条件
     * @return
     */
    @Override
    public Page queryByCondition(Page page, Wrapper<ObjectInfoCollect> queryWrapper) {
        //查询交付范围内信息点
        Page selectPage = objectInfoCollectMapper.selectPage(page, queryWrapper);
        List<ObjectInfoCollect> records =(List<ObjectInfoCollect>)selectPage.getRecords();
         if(CollUtil.isEmpty(records)){
            return selectPage;
        }
        Map<String, ObjectInfoDefine> infoMap = queryDefineObjectInfo(records);
        List<ObjectInfoCollectDTO> content = new ArrayList<>();
        records.forEach(item->{
            ObjectInfoCollectDTO collectDTO = new ObjectInfoCollectDTO();
            String defineInfoId = item.getDefineInfoId();
            //设置标准信息点信息
            if(StringUtils.isNotEmpty(defineInfoId)){
                ObjectInfoDefine objectInfoDefine = infoMap.get(defineInfoId);
                if(objectInfoDefine != null){
                    BeanUtils.copyProperties(objectInfoDefine,collectDTO);
                }
            }
            BeanUtils.copyProperties(item,collectDTO);
            content.add(collectDTO);
        });
        selectPage.setRecords(content);
        return selectPage;
    }

    /**
     * 查询标准信息点
     * @return
     */
    private Map<String, ObjectInfoDefine> queryDefineObjectInfo(List<ObjectInfoCollect> records){
        Set<String> ids = records.stream().map(ObjectInfoCollect::getDefineInfoId).collect(Collectors.toSet());
        QueryWrapper<ObjectInfoDefine> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id",ids);
        //queryWrapper.select("id","name");
        List<ObjectInfoDefine> objectInfoDefines = objectInfoService.queryByCondition(queryWrapper);
        if(CollUtil.isEmpty(objectInfoDefines)){
            return new HashMap<>();
        }
        return objectInfoDefines.stream().collect(Collectors.toMap(ObjectInfoDefine::getId, item->item, (k1, k2) -> k1));
    }

    /**
     * 新增采集的信息点
     * @param voList 待新增数据
     * @return
     */
    @Override
    public List<ObjectInfoCollect> insert(List<ObjectInfoCollect> voList) throws Exception{
        //填充信息点采集ID
        fillObjectInfoCollectId(voList);
        //新增采集信息点
        saveOrUpdateBatch(voList);
        return voList;
    }

    /**
     * 修改采集的信息点
     * @param voList 待修改数据
     * @return
     */
    @Override
    public List<ObjectInfoCollect> update(List<ObjectInfoCollect> voList) throws Exception {
        //填充信息点采集ID
        fillObjectInfoCollectId(voList);
        //修改采集信息点
        saveOrUpdateBatch(voList);
        return voList;
    }

    /**
     * 删除采集的信息点
     * @param idList 主键
     */
    @Override
    public void delete(List<String> idList) {
        if(CollUtil.isEmpty(idList)) {
            return;
        }
        List<ObjectInfoCollect> dbList = objectInfoCollectMapper.selectBatchIds(idList);
        Map<String, ObjectInfoCollect> dbMap = CollectionUtil.fieldValueMap(dbList, BaseEntity.PROP_ID);
        idList.forEach(id -> {
            ObjectInfoCollect dbVO = MapUtil.get(dbMap, id, ObjectInfoCollect.class);
            // 容错处理
            if(dbVO == null) {
                return;
            }
            // 删除
            ObjectInfoCollect vo = new ObjectInfoCollect();
            vo.setId(id);
            vo.setValid(ValidEnum.FALSE.getType());
            objectInfoCollectMapper.updateById(vo);
        });
    }

    /**
     * 校验信息点是否已被采集，若已被采集，填充相应的采集ID
     * @param voList
     */
    private void fillObjectInfoCollectId(List<ObjectInfoCollect> voList){
        if(CollUtil.isEmpty(voList)){
            return;
        }
        for (ObjectInfoCollect objectInfoCollect : voList) {
            if(StringUtils.isNotEmpty(objectInfoCollect.getId()) || StringUtils.isEmpty(objectInfoCollect.getDefineInfoId())){
                continue;
            }
            //查询采集信息点
            LambdaQueryWrapper<ObjectInfoCollect> wrapper = new LambdaQueryWrapper<>();
            wrapper.eq(ObjectInfoCollect::getGroupCode,objectInfoCollect.getGroupCode());
            wrapper.eq(ObjectInfoCollect::getProjectId,objectInfoCollect.getProjectId());
            wrapper.eq(ObjectInfoCollect::getDefineInfoId,objectInfoCollect.getDefineInfoId());
            wrapper.eq(ObjectInfoCollect::getClassCode,objectInfoCollect.getClassCode());
            wrapper.eq(ObjectInfoCollect::getValid,true);
            List<ObjectInfoCollect> objectInfoCollects = queryByCondition(wrapper);
            if(CollUtil.isEmpty(objectInfoCollects)){
                //信息点未被采集
                continue;
            }else{
                //信息点已采集
                objectInfoCollect.setId(objectInfoCollects.get(0).getId());
            }
        }
    }
}
